@@ -3,12 +3,60 @@
 
 #define _FILE_OFFSET_BITS 64
 
+#include <errno.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "lzip.h"
 #include "LzmaDec.h"
 
-#include <string.h>
+
+CRC32 crc32;
+
+
+/* Returns the number of bytes really read.
+   If (returned value < size) and (errno == 0), means EOF was reached.
+*/
+int readblock( const int fd, uint8_t * const buf, const int size )
+  {
+  int rest = size;
+  errno = 0;
+  while( rest > 0 )
+    {
+    const int n = read( fd, buf + size - rest, rest );
+    if( n > 0 ) rest -= n;
+    else if( n == 0 ) break;				/* EOF */
+    else if( errno != EINTR && errno != EAGAIN ) break;
+    errno = 0;
+    }
+  return size - rest;
+  }
+
+
+/* Returns the number of bytes really written.
+   If (returned value < size), it is always an error.
+*/
+int writeblock( const int fd, const uint8_t * const buf, const int size )
+  {
+  int rest = size;
+  errno = 0;
+  while( rest > 0 )
+    {
+    const int n = write( fd, buf + size - rest, rest );
+    if( n > 0 ) rest -= n;
+    else if( n < 0 && errno != EINTR && errno != EAGAIN ) break;
+    errno = 0;
+    }
+  return size - rest;
+  }
+
 
 #define kNumTopBits 24
-#define kTopValue ((UInt32)1 << kNumTopBits)
+#define kTopValue ((uint32_t)1 << kNumTopBits)
 
 #define kNumBitModelTotalBits 11
 #define kBitModelTotal (1 << kNumBitModelTotalBits)
@@ -19,8 +67,8 @@
 #define NORMALIZE if (range < kTopValue) { range <<= 8; code = (code << 8) | (*buf++); }
 
 #define IF_BIT_0(p) ttt = *(p); NORMALIZE; bound = (range >> kNumBitModelTotalBits) * ttt; if (code < bound)
-#define UPDATE_0(p) range = bound; *(p) = (CLzmaProb)(ttt + ((kBitModelTotal - ttt) >> kNumMoveBits));
-#define UPDATE_1(p) range -= bound; code -= bound; *(p) = (CLzmaProb)(ttt - (ttt >> kNumMoveBits));
+#define UPDATE_0(p) range = bound; *(p) = (int)(ttt + ((kBitModelTotal - ttt) >> kNumMoveBits));
+#define UPDATE_1(p) range -= bound; code -= bound; *(p) = (int)(ttt - (ttt >> kNumMoveBits));
 #define GET_BIT2(p, i, A0, A1) IF_BIT_0(p) \
   { UPDATE_0(p); i = (i + i); A0; } else \
   { UPDATE_1(p); i = (i + i) + 1; A1; }
@@ -109,20 +157,19 @@
 #define LZMA_BASE_SIZE 1846
 #define LZMA_LIT_SIZE 768
 
-#define LzmaProps_GetNumProbs(p) ((UInt32)LZMA_BASE_SIZE + (LZMA_LIT_SIZE << ((p)->lc + (p)->lp)))
+#define LzmaProps_GetNumProbs(p) ((uint32_t)LZMA_BASE_SIZE + (LZMA_LIT_SIZE << ((p)->lc + (p)->lp)))
 
 #if Literal != LZMA_BASE_SIZE
 StopCompilingDueBUG
 #endif
 
-#define LZMA_DIC_MIN (1 << 12)
 
 /* First LZMA-symbol is always decoded.
 And it decodes new LZMA-symbols while (buf < bufLimit), but "buf" is without last normalization
 Out:
   Result:
-    SZ_OK - OK
-    SZ_ERROR_DATA - Error
+    true - OK
+    false - Error
   p->remainLen:
     < kMatchSpecLenStart : normal remain
     = kMatchSpecLenStart : finished
@@ -130,32 +177,32 @@ Out:
     = kMatchSpecLenStart + 2 : State Init Marker
 */
 
-static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte *bufLimit)
+static bool LzmaDec_DecodeReal(CLzmaDec *p, uint32_t limit, const uint8_t *bufLimit)
 {
-  CLzmaProb *probs = p->probs;
+  int *probs = p->probs;
 
-  unsigned state = p->state;
-  UInt32 rep0 = p->reps[0], rep1 = p->reps[1], rep2 = p->reps[2], rep3 = p->reps[3];
-  unsigned pbMask = ((unsigned)1 << (p->prop.pb)) - 1;
-  unsigned lpMask = ((unsigned)1 << (p->prop.lp)) - 1;
-  unsigned lc = p->prop.lc;
+  State state = p->state;
+  uint32_t rep0 = p->reps[0], rep1 = p->reps[1], rep2 = p->reps[2], rep3 = p->reps[3];
+  unsigned pbMask = ((unsigned)1 << (p->pb)) - 1;
+  unsigned lpMask = ((unsigned)1 << (p->lp)) - 1;
+  const unsigned lc = p->lc;
 
-  Byte *dic = p->dic;
-  SizeT dicBufSize = p->dicBufSize;
-  SizeT dicPos = p->dicPos;
+  uint8_t *dic = p->dic;
+  const uint32_t dicBufSize = p->dicBufSize;
+  uint32_t dicPos = p->dicPos;
 
-  UInt32 processedPos = p->processedPos;
-  UInt32 checkDicSize = p->checkDicSize;
+  uint32_t processedPos = p->processedPos;
+  uint32_t checkDicSize = p->checkDicSize;
   unsigned len = 0;
 
-  const Byte *buf = p->buf;
-  UInt32 range = p->range;
-  UInt32 code = p->code;
+  const uint8_t *buf = p->buf;
+  uint32_t range = p->range;
+  uint32_t code = p->code;
 
   do
   {
-    CLzmaProb *prob;
-    UInt32 bound;
+    int *prob;
+    uint32_t bound;
     unsigned ttt;
     unsigned posState = processedPos & pbMask;
 
@@ -184,7 +231,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
         do
         {
           unsigned bit;
-          CLzmaProb *probLit;
+          int *probLit;
           matchByte <<= 1;
           bit = (matchByte & offs);
           probLit = prob + offs + bit + symbol;
@@ -192,7 +239,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
         }
         while (symbol < 0x100);
       }
-      dic[dicPos++] = (Byte)symbol;
+      dic[dicPos++] = (uint8_t)symbol;
       processedPos++;
       continue;
     }
@@ -210,7 +257,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
       {
         UPDATE_1(prob);
         if (checkDicSize == 0 && processedPos == 0)
-          return SZ_ERROR_DATA;
+          return false;
         prob = probs + IsRepG0 + state;
         IF_BIT_0(prob)
         {
@@ -229,7 +276,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
         }
         else
         {
-          UInt32 distance;
+          uint32_t distance;
           UPDATE_1(prob);
           prob = probs + IsRepG1 + state;
           IF_BIT_0(prob)
@@ -262,7 +309,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
       }
       {
         unsigned limit, offset;
-        CLzmaProb *probLen = prob + LenChoice;
+        int *probLen = prob + LenChoice;
         IF_BIT_0(probLen)
         {
           UPDATE_0(probLen);
@@ -295,7 +342,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
 
       if (state >= kNumStates)
       {
-        UInt32 distance;
+        uint32_t distance;
         prob = probs + PosSlot +
             ((len < kNumLenToPosStates ? len : kNumLenToPosStates - 1) << kNumPosSlotBits);
         TREE_6_DECODE(prob, distance);
@@ -309,7 +356,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
             distance <<= numDirectBits;
             prob = probs + SpecPos + distance - posSlot - 1;
             {
-              UInt32 mask = 1;
+              uint32_t mask = 1;
               unsigned i = 1;
               do
               {
@@ -328,9 +375,9 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
               range >>= 1;
 
               {
-                UInt32 t;
+                uint32_t t;
                 code -= range;
-                t = (0 - ((UInt32)code >> 31)); /* (UInt32)((Int32)code >> 31) */
+                t = (0 - ((uint32_t)code >> 31)); /* (uint32_t)((int)code >> 31) */
                 distance = (distance << 1) + (t + 1);
                 code += range & t;
               }
@@ -353,7 +400,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
               GET_BIT2(prob + i, i, ; , distance |= 4);
               GET_BIT2(prob + i, i, ; , distance |= 8);
             }
-            if (distance == (UInt32)0xFFFFFFFF)
+            if (distance == (uint32_t)0xFFFFFFFF)
             {
               len += kMatchSpecLenStart;
               state -= kNumStates;
@@ -368,33 +415,33 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
         if (checkDicSize == 0)
         {
           if (distance >= processedPos)
-            return SZ_ERROR_DATA;
+            return false;
         }
         else if (distance >= checkDicSize)
-          return SZ_ERROR_DATA;
+          return false;
         state = (state < kNumStates + kNumLitStates) ? kNumLitStates : kNumLitStates + 3;
       }
 
       len += kMatchMinLen;
 
       if (limit == dicPos)
-        return SZ_ERROR_DATA;
+        return false;
       {
-        SizeT rem = limit - dicPos;
+        uint32_t rem = limit - dicPos;
         unsigned curLen = ((rem < len) ? (unsigned)rem : len);
-        SizeT pos = (dicPos - rep0) + ((dicPos < rep0) ? dicBufSize : 0);
+        uint32_t pos = (dicPos - rep0) + ((dicPos < rep0) ? dicBufSize : 0);
 
         processedPos += curLen;
 
         len -= curLen;
         if (pos + curLen <= dicBufSize)
         {
-          Byte *dest = dic + dicPos;
+          uint8_t *dest = dic + dicPos;
           ptrdiff_t src = (ptrdiff_t)pos - (ptrdiff_t)dicPos;
-          const Byte *lim = dest + curLen;
+          const uint8_t *lim = dest + curLen;
           dicPos += curLen;
           do
-            *(dest) = (Byte)*(dest + src);
+            *(dest) = (uint8_t)*(dest + src);
           while (++dest != lim);
         }
         else
@@ -424,23 +471,23 @@ static int MY_FAST_CALL LzmaDec_DecodeReal(CLzmaDec *p, SizeT limit, const Byte
   p->reps[3] = rep3;
   p->state = state;
 
-  return SZ_OK;
+  return true;
 }
 
-static void MY_FAST_CALL LzmaDec_WriteRem(CLzmaDec *p, SizeT limit)
+static void LzmaDec_WriteRem(CLzmaDec *p, uint32_t limit)
 {
   if (p->remainLen != 0 && p->remainLen < kMatchSpecLenStart)
   {
-    Byte *dic = p->dic;
-    SizeT dicPos = p->dicPos;
-    SizeT dicBufSize = p->dicBufSize;
+    uint8_t *dic = p->dic;
+    uint32_t dicPos = p->dicPos;
+    const uint32_t dicBufSize = p->dicBufSize;
     unsigned len = p->remainLen;
-    UInt32 rep0 = p->reps[0];
+    uint32_t rep0 = p->reps[0];
     if (limit - dicPos < len)
       len = (unsigned)(limit - dicPos);
 
-    if (p->checkDicSize == 0 && p->prop.dicSize - p->processedPos <= len)
-      p->checkDicSize = p->prop.dicSize;
+    if (p->checkDicSize == 0 && dicBufSize - p->processedPos <= len)
+      p->checkDicSize = dicBufSize;
 
     p->processedPos += len;
     p->remainLen -= len;
@@ -453,20 +500,21 @@ static void MY_FAST_CALL LzmaDec_WriteRem(CLzmaDec *p, SizeT limit)
   }
 }
 
-static int MY_FAST_CALL LzmaDec_DecodeReal2(CLzmaDec *p, SizeT limit, const Byte *bufLimit)
+static int LzmaDec_DecodeReal2(CLzmaDec *p, uint32_t limit, const uint8_t *bufLimit)
 {
+  const uint32_t dicBufSize = p->dicBufSize;
   do
   {
-    SizeT limit2 = limit;
+    uint32_t limit2 = limit;
     if (p->checkDicSize == 0)
     {
-      UInt32 rem = p->prop.dicSize - p->processedPos;
+      uint32_t rem = dicBufSize - p->processedPos;
       if (limit - p->dicPos > rem)
         limit2 = p->dicPos + rem;
     }
-    RINOK(LzmaDec_DecodeReal(p, limit2, bufLimit));
-    if (p->processedPos >= p->prop.dicSize)
-      p->checkDicSize = p->prop.dicSize;
+    if( !LzmaDec_DecodeReal(p, limit2, bufLimit) ) return false;
+    if (p->processedPos >= dicBufSize)
+      p->checkDicSize = dicBufSize;
     LzmaDec_WriteRem(p, limit);
   }
   while (p->dicPos < limit && p->buf < bufLimit && p->remainLen < kMatchSpecLenStart);
@@ -475,7 +523,7 @@ static int MY_FAST_CALL LzmaDec_DecodeReal2(CLzmaDec *p, SizeT limit, const Byte
   {
     p->remainLen = kMatchSpecLenStart;
   }
-  return 0;
+  return true;
 }
 
 typedef enum
@@ -486,20 +534,20 @@ typedef enum
   DUMMY_REP
 } ELzmaDummy;
 
-static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const Byte *buf, SizeT inSize)
+static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const uint8_t *buf, uint32_t inSize)
 {
-  UInt32 range = p->range;
-  UInt32 code = p->code;
-  const Byte *bufLimit = buf + inSize;
-  CLzmaProb *probs = p->probs;
-  unsigned state = p->state;
+  uint32_t range = p->range;
+  uint32_t code = p->code;
+  const uint8_t *bufLimit = buf + inSize;
+  int *probs = p->probs;
+  State state = p->state;
   ELzmaDummy res;
 
   {
-    CLzmaProb *prob;
-    UInt32 bound;
+    int *prob;
+    uint32_t bound;
     unsigned ttt;
-    unsigned posState = (p->processedPos) & ((1 << p->prop.pb) - 1);
+    unsigned posState = (p->processedPos) & ((1 << p->pb) - 1);
 
     prob = probs + IsMatch + (state << kNumPosBitsMax) + posState;
     IF_BIT_0_CHECK(prob)
@@ -511,8 +559,8 @@ static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const Byte *buf, SizeT inS
       prob = probs + Literal;
       if (p->checkDicSize != 0 || p->processedPos != 0)
         prob += (LZMA_LIT_SIZE *
-          ((((p->processedPos) & ((1 << (p->prop.lp)) - 1)) << p->prop.lc) +
-          (p->dic[(p->dicPos == 0 ? p->dicBufSize : p->dicPos) - 1] >> (8 - p->prop.lc))));
+          ((((p->processedPos) & ((1 << (p->lp)) - 1)) << p->lc) +
+          (p->dic[(p->dicPos == 0 ? p->dicBufSize : p->dicPos) - 1] >> (8 - p->lc))));
 
       if (state < kNumLitStates)
       {
@@ -528,7 +576,7 @@ static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const Byte *buf, SizeT inS
         do
         {
           unsigned bit;
-          CLzmaProb *probLit;
+          int *probLit;
           matchByte <<= 1;
           bit = (matchByte & offs);
           probLit = prob + offs + bit + symbol;
@@ -598,7 +646,7 @@ static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const Byte *buf, SizeT inS
       }
       {
         unsigned limit, offset;
-        CLzmaProb *probLen = prob + LenChoice;
+        int *probLen = prob + LenChoice;
         IF_BIT_0_CHECK(probLen)
         {
           UPDATE_0_CHECK;
@@ -677,51 +725,19 @@ static ELzmaDummy LzmaDec_TryDummy(const CLzmaDec *p, const Byte *buf, SizeT inS
 }
 
 
-static void LzmaDec_InitRc(CLzmaDec *p, const Byte *data)
+static void LzmaDec_InitRc(CLzmaDec *p, const uint8_t *data)
 {
-  p->code = ((UInt32)data[1] << 24) | ((UInt32)data[2] << 16) | ((UInt32)data[3] << 8) | ((UInt32)data[4]);
+  p->code = ((uint32_t)data[1] << 24) | ((uint32_t)data[2] << 16) | ((uint32_t)data[3] << 8) | ((uint32_t)data[4]);
   p->range = 0xFFFFFFFF;
-  p->needFlush = 0;
+  p->needFlush = false;
 }
 
-void LzmaDec_InitDicAndState(CLzmaDec *p, Bool initDic, Bool initState)
-{
-  p->needFlush = 1;
-  p->remainLen = 0;
-  p->tempBufSize = 0;
 
-  if (initDic)
-  {
-    p->processedPos = 0;
-    p->checkDicSize = 0;
-    p->needInitState = 1;
-  }
-  if (initState)
-    p->needInitState = 1;
-}
-
-void LzmaDec_Init(CLzmaDec *p)
+static bool LzmaDec_DecodeToDic(CLzmaDec *p, uint32_t dicLimit,
+                                const uint8_t *src, uint32_t *srcLen,
+                                ELzmaFinishMode finishMode, ELzmaStatus *status)
 {
-  p->dicPos = 0;
-  LzmaDec_InitDicAndState(p, True, True);
-}
-
-static void LzmaDec_InitStateReal(CLzmaDec *p)
-{
-  UInt32 numProbs = Literal + ((UInt32)LZMA_LIT_SIZE << (p->prop.lc + p->prop.lp));
-  UInt32 i;
-  CLzmaProb *probs = p->probs;
-  for (i = 0; i < numProbs; i++)
-    probs[i] = kBitModelTotal >> 1;
-  p->reps[0] = p->reps[1] = p->reps[2] = p->reps[3] = 1;
-  p->state = 0;
-  p->needInitState = 0;
-}
-
-SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *srcLen,
-    ELzmaFinishMode finishMode, ELzmaStatus *status)
-{
-  SizeT inSize = *srcLen;
+  uint32_t inSize = *srcLen;
   (*srcLen) = 0;
   LzmaDec_WriteRem(p, dicLimit);
 
@@ -731,17 +747,17 @@ SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *sr
   {
       int checkEndMarkNow;
 
-      if (p->needFlush != 0)
+      if( p->needFlush )
       {
         for (; inSize > 0 && p->tempBufSize < RC_INIT_SIZE; (*srcLen)++, inSize--)
           p->tempBuf[p->tempBufSize++] = *src++;
         if (p->tempBufSize < RC_INIT_SIZE)
         {
           *status = LZMA_STATUS_NEEDS_MORE_INPUT;
-          return SZ_OK;
+          return true;
         }
         if (p->tempBuf[0] != 0)
-          return SZ_ERROR_DATA;
+          return false;
 
         LzmaDec_InitRc(p, p->tempBuf);
         p->tempBufSize = 0;
@@ -753,28 +769,25 @@ SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *sr
         if (p->remainLen == 0 && p->code == 0)
         {
           *status = LZMA_STATUS_MAYBE_FINISHED_WITHOUT_MARK;
-          return SZ_OK;
+          return true;
         }
         if (finishMode == LZMA_FINISH_ANY)
         {
           *status = LZMA_STATUS_NOT_FINISHED;
-          return SZ_OK;
+          return true;
         }
         if (p->remainLen != 0)
         {
           *status = LZMA_STATUS_NOT_FINISHED;
-          return SZ_ERROR_DATA;
+          return false;
         }
         checkEndMarkNow = 1;
       }
 
-      if (p->needInitState)
-        LzmaDec_InitStateReal(p);
-
       if (p->tempBufSize == 0)
       {
-        SizeT processed;
-        const Byte *bufLimit;
+        uint32_t processed;
+        const uint8_t *bufLimit;
         if (inSize < LZMA_REQUIRED_INPUT_MAX || checkEndMarkNow)
         {
           int dummyRes = LzmaDec_TryDummy(p, src, inSize);
@@ -784,21 +797,21 @@ SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *sr
             p->tempBufSize = (unsigned)inSize;
             (*srcLen) += inSize;
             *status = LZMA_STATUS_NEEDS_MORE_INPUT;
-            return SZ_OK;
+            return true;
           }
           if (checkEndMarkNow && dummyRes != DUMMY_MATCH)
           {
             *status = LZMA_STATUS_NOT_FINISHED;
-            return SZ_ERROR_DATA;
+            return false;
           }
           bufLimit = src;
         }
         else
           bufLimit = src + inSize - LZMA_REQUIRED_INPUT_MAX;
         p->buf = src;
-        if (LzmaDec_DecodeReal2(p, dicLimit, bufLimit) != 0)
-          return SZ_ERROR_DATA;
-        processed = (SizeT)(p->buf - src);
+        if( !LzmaDec_DecodeReal2(p, dicLimit, bufLimit) )
+          return false;
+        processed = (uint32_t)(p->buf - src);
         (*srcLen) += processed;
         src += processed;
         inSize -= processed;
@@ -816,17 +829,17 @@ SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *sr
           {
             (*srcLen) += lookAhead;
             *status = LZMA_STATUS_NEEDS_MORE_INPUT;
-            return SZ_OK;
+            return true;
           }
           if (checkEndMarkNow && dummyRes != DUMMY_MATCH)
           {
             *status = LZMA_STATUS_NOT_FINISHED;
-            return SZ_ERROR_DATA;
+            return false;
           }
         }
         p->buf = p->tempBuf;
-        if (LzmaDec_DecodeReal2(p, dicLimit, p->buf) != 0)
-          return SZ_ERROR_DATA;
+        if( !LzmaDec_DecodeReal2(p, dicLimit, p->buf) )
+          return false;
         lookAhead -= (rem - (unsigned)(p->buf - p->tempBuf));
         (*srcLen) += lookAhead;
         src += lookAhead;
@@ -836,19 +849,21 @@ SRes LzmaDec_DecodeToDic(CLzmaDec *p, SizeT dicLimit, const Byte *src, SizeT *sr
   }
   if (p->code == 0)
     *status = LZMA_STATUS_FINISHED_WITH_MARK;
-  return (p->code == 0) ? SZ_OK : SZ_ERROR_DATA;
+  return (p->code == 0);
 }
 
-SRes LzmaDec_DecodeToBuf(CLzmaDec *p, Byte *dest, SizeT *destLen, const Byte *src, SizeT *srcLen, ELzmaFinishMode finishMode, ELzmaStatus *status)
+bool LzmaDec_DecodeToBuf( CLzmaDec *p, uint8_t *dest, uint32_t *destLen,
+                          const uint8_t *src, uint32_t *srcLen,
+                          ELzmaFinishMode finishMode, ELzmaStatus *status )
 {
-  SizeT outSize = *destLen;
-  SizeT inSize = *srcLen;
+  uint32_t outSize = *destLen;
+  uint32_t inSize = *srcLen;
   *srcLen = *destLen = 0;
   for (;;)
   {
-    SizeT inSizeCur = inSize, outSizeCur, dicPos;
+    uint32_t inSizeCur = inSize, outSizeCur, dicPos;
     ELzmaFinishMode curFinishMode;
-    SRes res;
+    bool res;
     if (p->dicPos == p->dicBufSize)
       p->dicPos = 0;
     dicPos = p->dicPos;
@@ -872,89 +887,52 @@ SRes LzmaDec_DecodeToBuf(CLzmaDec *p, Byte *dest, SizeT *destLen, const Byte *sr
     dest += outSizeCur;
     outSize -= outSizeCur;
     *destLen += outSizeCur;
-    if (res != 0)
-      return res;
+    if( !res )
+      return false;
     if (outSizeCur == 0 || outSize == 0)
-      return SZ_OK;
+      return true;
   }
 }
 
-void LzmaDec_FreeProbs(CLzmaDec *p, ISzAlloc *alloc)
+
+void LzmaDec_Free(CLzmaDec *p)
 {
-  alloc->Free(alloc, p->probs);
-  p->probs = 0;
+  free( p->dic );
+  free( p->probs );
 }
 
-static void LzmaDec_FreeDict(CLzmaDec *p, ISzAlloc *alloc)
+
+bool LzmaDec_Init(CLzmaDec *p, const uint8_t *raw_props)
 {
-  alloc->Free(alloc, p->dic);
-  p->dic = 0;
-}
-
-void LzmaDec_Free(CLzmaDec *p, ISzAlloc *alloc)
-{
-  LzmaDec_FreeProbs(p, alloc);
-  LzmaDec_FreeDict(p, alloc);
-}
-
-SRes LzmaProps_Decode(CLzmaProps *p, const Byte *data, unsigned size)
-{
-  UInt32 dicSize;
-  Byte d;
-
-  if (size < LZMA_PROPS_SIZE)
-    return SZ_ERROR_UNSUPPORTED;
-  else
-    dicSize = data[1] | ((UInt32)data[2] << 8) | ((UInt32)data[3] << 16) | ((UInt32)data[4] << 24);
-
-  if (dicSize < LZMA_DIC_MIN)
-    dicSize = LZMA_DIC_MIN;
-  p->dicSize = dicSize;
-
-  d = data[0];
-  if (d >= (9 * 5 * 5))
-    return SZ_ERROR_UNSUPPORTED;
+  uint32_t i;
+  uint8_t d = raw_props[0];
 
   p->lc = d % 9;
   d /= 9;
   p->pb = d / 5;
   p->lp = d % 5;
 
-  return SZ_OK;
-}
+  p->dicBufSize = raw_props[1] | ((uint32_t)raw_props[2] << 8) |
+                  ((uint32_t)raw_props[3] << 16) | ((uint32_t)raw_props[4] << 24);
+  if (p->dicBufSize < min_dictionary_size) p->dicBufSize = min_dictionary_size;
 
-static SRes LzmaDec_AllocateProbs2(CLzmaDec *p, const CLzmaProps *propNew, ISzAlloc *alloc)
-{
-  UInt32 numProbs = LzmaProps_GetNumProbs(propNew);
-  if (p->probs == 0 || numProbs != p->numProbs)
-  {
-    LzmaDec_FreeProbs(p, alloc);
-    p->probs = (CLzmaProb *)alloc->Alloc(alloc, numProbs * sizeof(CLzmaProb));
-    p->numProbs = numProbs;
-    if (p->probs == 0)
-      return SZ_ERROR_MEM;
-  }
-  return SZ_OK;
-}
-
-SRes LzmaDec_Allocate(CLzmaDec *p, const Byte *props, unsigned propsSize, ISzAlloc *alloc)
-{
-  CLzmaProps propNew;
-  SizeT dicBufSize;
-  RINOK(LzmaProps_Decode(&propNew, props, propsSize));
-  RINOK(LzmaDec_AllocateProbs2(p, &propNew, alloc));
-  dicBufSize = propNew.dicSize;
-  if (p->dic == 0 || dicBufSize != p->dicBufSize)
-  {
-    LzmaDec_FreeDict(p, alloc);
-    p->dic = (Byte *)alloc->Alloc(alloc, dicBufSize);
-    if (p->dic == 0)
+  p->numProbs = LzmaProps_GetNumProbs(p);
+  p->probs = (int *)malloc(p->numProbs * sizeof(int));
+  if( !p->probs ) return false;
+  p->dic = (uint8_t *)malloc(p->dicBufSize);
+  if (p->dic == 0)
     {
-      LzmaDec_FreeProbs(p, alloc);
-      return SZ_ERROR_MEM;
+    free( p->probs );
+    return false;
     }
-  }
-  p->dicBufSize = dicBufSize;
-  p->prop = propNew;
-  return SZ_OK;
+  p->dicPos = 0;
+  p->needFlush = true;
+  p->remainLen = 0;
+  p->tempBufSize = 0;
+  p->processedPos = 0;
+  p->checkDicSize = 0;
+  for( i = 0; i < p->numProbs; ++i ) p->probs[i] = kBitModelTotal >> 1;
+  p->reps[0] = p->reps[1] = p->reps[2] = p->reps[3] = 1;
+  p->state = 0;
+  return true;
 }
